@@ -1,5 +1,5 @@
 """
-Copyright (C) 2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+Copyright (C) 2021-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
 
 This module provides a tool to download and update clamav databases and database
 patch files (CDIFFs) for the purposes of hosting your own database mirror.
@@ -24,6 +24,7 @@ from enum import Enum
 import json
 import logging
 import os
+import platform
 import pkg_resources
 import re
 import subprocess
@@ -58,7 +59,10 @@ class CVDUpdate:
         "db directory" : str(Path.home() / ".cvdupdate" / "database"),
         "rotate cdiffs" : True,
         "# cdiffs to keep" : 30,
+        "state file": "",
+    }
 
+    default_state: dict = {
         "dbs" : {
             "main.cvd" : {
                 "url" : "https://database.clamav.net/main.cvd",
@@ -92,6 +96,7 @@ class CVDUpdate:
 
     config_path: Path
     config: dict
+    state: dict
     db_dir: Path
     log_dir: Path
     version: str
@@ -176,7 +181,7 @@ class CVDUpdate:
         need_save = False
 
         if config == "":
-            self.config_path = self.default_config_path
+            self.config_path = copy.deepcopy(self.default_config_path)
         else:
             self.config_path = Path(config)
 
@@ -186,12 +191,7 @@ class CVDUpdate:
                 self.config = json.load(config_file)
         else:
             # Config does not exist, use default
-            self.config = self.default_config
-            need_save = True
-
-        if 'uuid' not in self.config:
-            # Create a UUID to put in our User-Agent for better (anonymous) metrics
-            self.config['uuid'] = str(uuid.uuid4())
+            self.config = copy.deepcopy(self.default_config)
             need_save = True
 
         if db_dir != "":
@@ -216,6 +216,40 @@ class CVDUpdate:
             self.config['max retry'] = 3
             need_save = True
 
+        if not hasattr(self, 'state'):
+            self.state = {}
+
+        # keep database state in a separate file, defaulting to same dir as config file
+        if 'state file' not in self.config or self.config['state file'] == '':
+            self.config['state file'] = str(self.config_path.parent / "state.json")
+            need_save = True
+
+        # handle migration from config.json to state.json
+        if 'dbs' in self.config:
+            self.state['dbs'] = self.config['dbs']
+            del self.config['dbs']
+
+        if 'uuid' in self.config:
+            self.state['uuid'] = self.config['uuid']
+            del self.config['uuid']
+
+        state_file = Path(self.config['state file'])
+        if state_file.exists():
+            # state file exists, load it.
+            with state_file.open('r') as st_fi:
+                self.state = json.load(st_fi)
+        elif self.state == {} or 'dbs' not in self.state:
+            # state file does not exist
+            # so we either have a fresh install or we have a messed up json
+            # create a skeleton structure
+            self.state = copy.deepcopy(self.default_state)
+            need_save = True
+
+        if 'uuid' not in self.state:
+            # Create a UUID to put in our User-Agent for better (anonymous) metrics
+            self.state['uuid'] = str(uuid.uuid4())
+            need_save = True
+
         if need_save:
             self._save_config()
 
@@ -223,13 +257,14 @@ class CVDUpdate:
         """
         Save the current configuration.
         """
-        if not self.config_path.parent.exists():
-            # config directory doesn't exist yet
-            try:
-                os.makedirs(str(self.config_path.parent))
-            except Exception as exc:
-                print("Failed to create config directory!")
-                raise exc
+        for fi in (self.config_path, Path(self.config['state file'])):
+            if not fi.parent.exists():
+                # parent directory doesn't exist yet
+                try:
+                    os.makedirs(str(fi.parent))
+                except Exception as exc:
+                    print("Failed to create config directory!")
+                    raise exc
 
         try:
             with self.config_path.open('w') as config_file:
@@ -238,8 +273,16 @@ class CVDUpdate:
             print("Failed to create config file!")
             raise exc
 
+        try:
+            with open(self.config['state file'], 'w') as state_file:
+                json.dump(self.state, state_file, indent=4)
+        except Exception as exc:
+            print("Failed to create state file!")
+            raise exc
+
         if self.verbose:
             print(f"Saved: {self.config_path}\n")
+            print(f"Saved: {self.config['state file']}\n")
 
     def config_show(self):
         """
@@ -247,6 +290,8 @@ class CVDUpdate:
         """
         print(f"Config file: {self.config_path}\n")
         print(f"Config:\n{json.dumps(self.config, indent=4)}\n")
+        print(f"State file: {self.config['state file']}\n")
+        print(f"State:\n{json.dumps(self.state, indent=4)}\n")
 
     def update(self, db: str = "") -> bool:
         """
@@ -255,13 +300,37 @@ class CVDUpdate:
 
     def clean_dbs(self):
         """
-        Delete all files in the database directory.
+        Delete cvd controlled files in the database directory.
         """
-        self.logger.info(f"Deleting databases...")
-        dbs = self.db_dir.glob('*')
+        dbs = self.state['dbs'].keys()
         for db in dbs:
-            os.remove(str(db))
-            self.logger.info(f"Deleted: {db}")
+            cvddb = self.db_dir / db
+            if cvddb.exists():
+                try:
+                    self.logger.info(f"Deleting: {db}")
+                    os.remove(str(cvddb))
+                except Exception as exc:
+                    self.logger.debug(f"Tried to remove {db}")
+                    raise exc
+
+        # Remove / clear all CDIFFs
+        cdiff_files = self.db_dir.glob('*.cdiff')
+        for cdiff in cdiff_files:
+            try:
+                self.logger.info(f"Deleting CDIFF: {cdiff.name}")
+                os.remove(str(cdiff))
+            except Exception as exc:
+                self.logger.debug(f"Tried to remove cdiffs.")
+
+        # Config cleanup
+        for db in dbs:
+            self.state['dbs'][db]['CDIFFs'] = []
+            self.state['dbs'][db]['last modified'] = 0
+            self.state['dbs'][db]['last checked'] = 0
+            self.state['dbs'][db]['local version'] = 0
+
+        # Save config
+        self._save_config()
 
     def clean_logs(self):
         """
@@ -283,10 +352,12 @@ class CVDUpdate:
 
         os.remove(str(self.config_path))
         print(f"Deleted: {self.config_path}")
+        os.remove(str(self.config['state file']))
+        print(f"Deleted: {self.config['state file']}")
 
     def _index_local_databases(self) -> dict:
         need_save = False
-        dbs = copy.deepcopy(self.config['dbs'])
+        dbs = copy.deepcopy(self.state['dbs'])
 
         db_paths = self.db_dir.glob('*')
         for db in db_paths:
@@ -319,7 +390,7 @@ class CVDUpdate:
                 }
             else:
                 # DB on disk is from our config
-                if db.name.endswith(".cvd") and self.config['dbs'][db.name]['local version'] == 0:
+                if db.name.endswith(".cvd") and self.state['dbs'][db.name]['local version'] == 0:
                     # Seems like we somehow got a (config'd) CVD file in our database directory without
                     # saving the CVD info to the config. Let's just update the version field.
                     self.logger.info(f"Found {db.name} in the DB directory, though it wasn't downloaded using this tool.")
@@ -328,7 +399,7 @@ class CVDUpdate:
                         self.logger.info(f"Identified mysterious {db.name} version: {dbs[db.name]['local version']}")
 
                         # Add the version info for this mysteriously deposited CVD to our config.
-                        self.config['dbs'][db.name]['local version'] = dbs[db.name]['local version']
+                        self.state['dbs'][db.name]['local version'] = dbs[db.name]['local version']
                         need_save = True
                     except Exception as exc:
                         self.logger.debug(f"EXCEPTION OCCURRED: {exc}")
@@ -410,17 +481,13 @@ class CVDUpdate:
         try:
             our_resolver = resolver.Resolver()
             our_resolver.timeout = 5 # Explicitly setting query timeout to mitigate https://github.com/Cisco-Talos/cvdupdate/issues/17
-            nameserver = os.environ.get("CVDUPDATE_NAMESERVER")
+            nameservers = self._get_nameserver_configuration()
 
-            if nameserver != None:
-                # Override the default nameserver using the CVDUPDATE_NAMESERVER environment variable.
-                our_resolver.nameservers = [nameserver]
-                self.logger.info(f"Using nameserver specified in CVDUPDATE_NAMESERVER: {nameserver}")
-
-            elif self.config['nameserver'] != "":
-                # Override the default nameserver using a config setting.
-                our_resolver.nameservers = [self.config['nameserver']]
-                self.logger.info(f"Using nameserver specified in the config: {self.config['nameserver']}")
+            if nameservers:
+                our_resolver.nameservers = nameservers
+                self.logger.info(f"Using nameservers: {nameservers}")
+            else:
+                self.logger.info("Using system configured nameservers")
 
             answer = str(our_resolver.resolve("current.cvd.clamav.net","TXT").response.answer[0])
             versions = re.search('".*"', answer).group().strip('"')
@@ -432,6 +499,46 @@ class CVDUpdate:
 
         return got_it
 
+    def _get_nameserver_configuration(self) -> List[str]:
+        '''
+        Parse comma delimited nameserver string into a list for Resolver
+        '''
+        nameserver_string = self._get_nameserver_string()
+        nameservers = []
+
+        os_platform = platform.platform()
+        operating_system = os_platform.split("-")[0].lower()
+
+        if nameserver_string != "":
+            try:
+                nameservers = [x.strip() for x in nameserver_string.split(',')]
+            except Exception as exc:
+                self.logger.warning(f"Failed to parse nameserver configuration: {nameserver_string}, ignoring...")
+                nameservers = []
+
+        if nameservers == [] and operating_system == "windows":
+            # Try OpenDNS nameservers on Windows if not overridden by the user, because leaving it empty seems to fail.
+            nameservers = ['208.67.222.222', '208.67.220.220']
+
+        return nameservers
+
+    def _get_nameserver_string(self) -> str:
+        '''
+        Determine user provided nameserver configuration
+        '''
+        config_nameserver = self.config['nameserver']
+        env_nameserver = os.environ.get("CVDUPDATE_NAMESERVER")
+
+        # Environment variable overrides the configuration setting
+        if env_nameserver != None and env_nameserver != "":
+            self.logger.info(f"Found CVDUPDATE_NAMESERVER environment variable to provide nameservers: {env_nameserver}")
+            return env_nameserver
+        elif config_nameserver != None and config_nameserver != "":
+            self.logger.info(f"Found configuration provided nameservers: {config_nameserver}")
+            return config_nameserver
+
+        return ""
+
     def _query_cvd_version_dns(self, db: str) -> int:
         '''
         This is a faux query.
@@ -448,17 +555,17 @@ class CVDUpdate:
 
         self.logger.debug(f"Checking {db} version via DNS TXT advertisement.")
 
-        if self.config['dbs'][db]['DNS field'] == 0:
+        if self.state['dbs'][db]['DNS field'] == 0:
             # Invalid DNS field value for database version.
             self.logger.warning(f"Failed to get DB version from DNS TXT entry: Invalid DNS field value for database version.")
             return version
 
         try:
-            version = int(self.dns_version_tokens[self.config['dbs'][db]['DNS field']])
+            version = int(self.dns_version_tokens[self.state['dbs'][db]['DNS field']])
             self.logger.debug(f"{db} version advertised by DNS: {version}")
 
             # Update the "last checked" time.
-            self.config['dbs'][db]['last checked'] = time.time()
+            self.state['dbs'][db]['last checked'] = time.time()
 
         except Exception as exc:
             self.logger.debug(f"EXCEPTION OCCURRED: {exc}")
@@ -475,15 +582,15 @@ class CVDUpdate:
         '''
         version = 0
         retry = 0
-        url = self.config['dbs'][db]['url']
+        url = self.state['dbs'][db]['url']
 
         self.logger.debug(f"Checking {db} version via HTTP download of CVD header.")
 
-        ims = datetime.datetime.utcfromtimestamp(self.config['dbs'][db]['last modified']).strftime('%a, %d %b %Y %H:%M:%S GMT')
+        ims = datetime.datetime.utcfromtimestamp(self.state['dbs'][db]['last modified']).strftime('%a, %d %b %Y %H:%M:%S GMT')
 
         while retry < self.config['max retry']:
             response = requests.get(url, headers = {
-                'User-Agent': f'CVDUPDATE/{self.version} ({self.config["uuid"]})',
+                'User-Agent': f'CVDUPDATE/{self.version} ({self.state["uuid"]})',
                 'Range': 'bytes=0-95',
                 'If-Modified-Since': ims,
             })
@@ -513,7 +620,7 @@ class CVDUpdate:
         elif response.status_code == 304:
             # HTTP Not-Modified, it's not newer.than what we already have.
             # Just return the current local version.
-            version = self.config['dbs'][db]['local version']
+            version = self.state['dbs'][db]['local version']
             self.logger.debug(f"{db} not-modified since: {ims} (local version {version})")
 
         elif response.status_code == 429:
@@ -525,7 +632,7 @@ class CVDUpdate:
             if 'Retry-After' in response.headers.keys():
                 try_again_seconds = int(response.headers['Retry-After'])
 
-            self.config['dbs'][db]['retry after'] = time.time() + float(try_again_seconds)
+            self.state['dbs'][db]['retry after'] = time.time() + float(try_again_seconds)
 
             try_again_string = str(datetime.timedelta(seconds=try_again_seconds))
             self.logger.warning(f"We won't try {db} again for {try_again_string} hours.")
@@ -536,7 +643,7 @@ class CVDUpdate:
 
         if version > 0:
             # Update the "last checked" time.
-            self.config['dbs'][db]['last checked'] = time.time()
+            self.state['dbs'][db]['last checked'] = time.time()
 
         return version
 
@@ -551,7 +658,7 @@ class CVDUpdate:
 
         while retry < self.config['max retry']:
             response = requests.get(url, headers = {
-                'User-Agent': f'CVDUPDATE/{self.version} ({self.config["uuid"]})',
+                'User-Agent': f'CVDUPDATE/{self.version} ({self.state["uuid"]})',
                 'If-Modified-Since': ims,
             })
 
@@ -582,9 +689,9 @@ class CVDUpdate:
                     new_db.write(response.content)
 
                 # Update config w/ new db info
-                self.config['dbs'][db]['last modified'] = time.time()
+                self.state['dbs'][db]['last modified'] = time.time()
                 if db.endswith('.cvd'):
-                    self.config['dbs'][db]['local version'] = self._get_version_from_cvd_header(response.content[:96])
+                    self.state['dbs'][db]['local version'] = self._get_version_from_cvd_header(response.content[:96])
 
             except Exception as exc:
                 self.logger.debug(f"EXCEPTION OCCURRED: {exc}")
@@ -593,7 +700,7 @@ class CVDUpdate:
 
         elif response.status_code == 304:
             # Not modified since IMS. We have the latest version.
-            version = self.config['dbs'][db]['local version']
+            version = self.state['dbs'][db]['local version']
             self.logger.info(f"{db} not-modified since: {ims} (local version {version})")
             return CvdStatus.NO_UPDATE
 
@@ -606,7 +713,7 @@ class CVDUpdate:
             if 'Retry-After' in response.headers.keys():
                 try_again_seconds = int(response.headers['Retry-After'])
 
-            self.config['dbs'][db]['retry after'] = time.time() + float(try_again_seconds)
+            self.state['dbs'][db]['retry after'] = time.time() + float(try_again_seconds)
 
             try_again_string = str(datetime.timedelta(seconds=try_again_seconds))
             self.logger.warning(f"We won't try {db} again for {try_again_string} hours.")
@@ -627,7 +734,7 @@ class CVDUpdate:
         If we already have some version of the database, attempt to download all CDIFFs in between.
         If we don't, just get the last two CDIFFs.
         '''
-        local_version = self.config['dbs'][db]['local version']
+        local_version = self.state['dbs'][db]['local version']
         desired_version = local_version + 1
 
         if local_version >= available_version:
@@ -651,17 +758,19 @@ class CVDUpdate:
             retry = 0
             cdiff_filename = f"{db[:-len('.cvd')]}-{desired_version}.cdiff"
 
-            original_url = self.config['dbs'][db]['url']
+            original_url = self.state['dbs'][db]['url']
             url = f"{original_url[:-len(db)]}{cdiff_filename}"
 
             if (self.db_dir / cdiff_filename).exists():
                 self.logger.debug(f"We already have {cdiff_filename}. Skipping...")
+                desired_version += 1
+                continue
 
             self.logger.debug(f"Checking for {cdiff_filename}")
 
             while retry < self.config['max retry']:
                 response = requests.get(url, headers = {
-                    'User-Agent': f'CVDUPDATE/{self.version} ({self.config["uuid"]})',
+                    'User-Agent': f'CVDUPDATE/{self.version} ({self.state["uuid"]})',
                 })
 
                 if ((response.status_code == 200 or response.status_code == 206) and
@@ -690,17 +799,17 @@ class CVDUpdate:
                     self.logger.error(f"Failed to save {cdiff_filename} to {self.db_dir}.")
 
                 # Update config with CDIFF, for posterity
-                self.config['dbs'][db]['CDIFFs'].append(cdiff_filename)
+                self.state['dbs'][db]['CDIFFs'].append(cdiff_filename)
 
                 # Prune old CDIFFs if needed
-                if len(self.config['dbs'][db]['CDIFFs']) > self.config['# cdiffs to keep']:
+                if len(self.state['dbs'][db]['CDIFFs']) > self.config['# cdiffs to keep']:
                     try:
-                        os.remove(self.db_dir / self.config['dbs'][db]['CDIFFs'][0])
+                        os.remove(self.db_dir / self.state['dbs'][db]['CDIFFs'][0])
                     except Exception as exc:
                         self.logger.debug(f"EXCEPTION OCCURRED: {exc}")
                         self.logger.debug(f"Tried to prune old cdiffs, but they weren't found, maybe someone else removed them already.")
 
-                    self.config['dbs'][db]['CDIFFs'] = self.config['dbs'][db]['CDIFFs'][1:]
+                    self.state['dbs'][db]['CDIFFs'] = self.state['dbs'][db]['CDIFFs'][1:]
 
             elif response.status_code == 429:
                 # Rejected because downloading the same file too frequently.
@@ -711,7 +820,7 @@ class CVDUpdate:
                 if 'Retry-After' in response.headers.keys():
                     try_again_seconds = int(response.headers['Retry-After'])
 
-                self.config['dbs'][db]['retry after'] = time.time() + float(try_again_seconds)
+                self.state['dbs'][db]['retry after'] = time.time() + float(try_again_seconds)
 
                 try_again_string = str(datetime.timedelta(seconds=try_again_seconds))
                 self.logger.warning(f"We won't try {db} again for {try_again_string} hours.")
@@ -736,7 +845,7 @@ class CVDUpdate:
         # Now download the available version.
         desired_version = available_version
 
-        url = f"{self.config['dbs'][db]['url']}?version={desired_version}"
+        url = f"{self.state['dbs'][db]['url']}?version={desired_version}"
 
         return self._download_db_from_url(db, url, last_modified=0, version=desired_version)
 
@@ -807,7 +916,7 @@ class CVDUpdate:
                 self.logger.debug(f'cvdupdate is up-to-date: {current_version}.')
                 return True
             else:
-                self.logger.warning(f'You are running cvdupate version: {current_version}.')
+                self.logger.warning(f'You are running cvdupdate version: {current_version}.')
                 self.logger.warning(f'There is a newer version on PyPI: {latest_version}. Please update!')
                 return False
 
@@ -834,7 +943,7 @@ class CVDUpdate:
         self._query_dns_txt_entry()
         if self.dns_version_tokens == []:
             # Query failed. Bail out.
-            self.logger.error(f"Failed to update {db}. Missing or invalid URL: {self.config['dbs'][db]['url']}")
+            self.logger.error(f"Failed to update: DNS query failed.")
             return 1
 
         if debug_mode:
@@ -844,34 +953,39 @@ class CVDUpdate:
             '''
             Update a database
             '''
-            if self.config['dbs'][db]['retry after'] > 0:
-                cooldown_date = datetime.datetime.fromtimestamp(self.config['dbs'][db]['retry after']).strftime('%Y-%m-%d %H:%M:%S')
+            if self.state['dbs'][db]['retry after'] > 0:
+                cooldown_date = datetime.datetime.fromtimestamp(self.state['dbs'][db]['retry after']).strftime('%Y-%m-%d %H:%M:%S')
 
-                if self.config['dbs'][db]['retry after'] > time.time():
+                if self.state['dbs'][db]['retry after'] > time.time():
                     self.logger.warning(f"Skipping {db} which is on cooldown until {cooldown_date}")
                     return CvdStatus.ERROR
                 else:
                     # Cooldown expired. Ok to try again.
-                    self.config['dbs'][db]['retry after'] = 0
+                    self.state['dbs'][db]['retry after'] = 0
                     self.logger.info(f"{db} cooldown expired {cooldown_date}. OK to try again...")
 
-            if not self.config['dbs'][db]['url'].startswith('http'):
-                self.logger.error(f"Failed to update {db}. Missing or invalid URL: {self.config['dbs'][db]['url']}")
+            if not self.state['dbs'][db]['url'].startswith('http'):
+                self.logger.error(f"Failed to update {db}. Missing or invalid URL: {self.state['dbs'][db]['url']}")
                 return CvdStatus.ERROR
 
-            self.logger.debug(f"Checking {db} for update from {self.config['dbs'][db]['url']}")
+            self.logger.debug(f"Checking {db} for update from {self.state['dbs'][db]['url']}")
 
             if db.endswith('.cvd'):
                 # It's a CVD (official signed clamav database)
                 advertised_version = 0
 
-                if self.config['dbs'][db]['local version'] == 0 and (self.db_dir / db).exists():
-                    # Seems like we somehow got a CVD in our database directory without
-                    # saving the CVD info to the config. Let's just update the version field.
-                    self.config['dbs'][db]['local version'] = self._get_cvd_version_from_file(self.db_dir / db)
+                if (self.db_dir / db).exists():
+                    if self.state['dbs'][db]['local version'] == 0:
+                        # Seems like we somehow got a CVD in our database directory without
+                        # saving the CVD info to the config. Let's just update the version field.
+                        self.state['dbs'][db]['local version'] = self._get_cvd_version_from_file(self.db_dir / db)
+                else:
+                    if self.state['dbs'][db]['local version'] != 0:
+                        # We have a local version but no CVD in the database directory.
+                        # Maybe it was moved or deleted? Let's just reset the local version.
+                        self.state['dbs'][db]['local version'] = 0
 
-
-                if self.config['dbs'][db]['DNS field'] > 0:
+                if self.state['dbs'][db]['DNS field'] > 0:
                     # We can use the DNS TXT fields to check if our version is old.
                     advertised_version = self._query_cvd_version_dns(db)
 
@@ -881,7 +995,7 @@ class CVDUpdate:
 
                     # First, make sure no one tampered with the DNS field for
                     # main/daily/bytecode when using database.clamav.net
-                    if (('database.clamav.net' in self.config['dbs'][db]['url']) and
+                    if (('database.clamav.net' in self.state['dbs'][db]['url']) and
                         (db == 'main.cvd' or db == 'daily.cvd' or db == 'bytecode.cvd')):
                         self.logger.error(f'It appears that the "DNS field" in {self.config_path} for "{db}" was modified from the default.')
                         self.logger.error(f'Updating {db} from database.clamav.net requires DNS for the version check in order to conserve bandwidth.')
@@ -902,12 +1016,12 @@ class CVDUpdate:
                 # If Not-Modified, it will not replace the current database.
                 return self._download_db_from_url(
                     db,
-                    self.config['dbs'][db]['url'],
-                    self.config['dbs'][db]['last modified'])
+                    self.state['dbs'][db]['url'],
+                    self.state['dbs'][db]['last modified'])
 
         if db == "":
             # Update every DB.
-            for db in self.config['dbs']:
+            for db in self.state['dbs']:
                 status = update(db)
                 if status == CvdStatus.ERROR:
                     self.update_errors += 1
@@ -916,7 +1030,7 @@ class CVDUpdate:
 
         else:
             # Update a specific DB.
-            if db not in self.config['dbs']:
+            if db not in self.state['dbs']:
                 self.logger.error(f"Update failed. Unknown database: {db}")
             else:
                 status = update(db)
@@ -956,12 +1070,12 @@ class CVDUpdate:
             ]:
             self.logger.warning(f"{db} does not have valid clamav database file extension.")
 
-        if db in self.config['dbs']:
+        if db in self.state['dbs']:
             self.logger.info(f"Cannot add {db}, it is already in our list.")
             self.logger.info(f"Hint: Try `db list -V` or `db show {db}` for more information.")
             return False
 
-        self.config['dbs'][db] = {
+        self.state['dbs'][db] = {
             "url" : url,
             "retry after" : 0,
             "last modified" : 0,
@@ -982,7 +1096,7 @@ class CVDUpdate:
         """
         Remove a database from our list, and delete copies of the DB from the database directory.
         """
-        if db not in self.config['dbs']:
+        if db not in self.state['dbs']:
             self.logger.info(f"Cannot remove {db}, it is not in our list.")
             self.logger.info(f"Hint: Try `db list -V` for more information.")
             return False
@@ -996,7 +1110,7 @@ class CVDUpdate:
             self.logger.debug(f"An exception occured: {exc}")
             self.logger.error(f"Failed to delete {db} from databse directory!")
 
-        for cdiff in self.config['dbs'][db]['CDIFFs']:
+        for cdiff in self.state['dbs'][db]['CDIFFs']:
             try:
                 if (self.db_dir / cdiff).exists():
                     os.remove(str(self.db_dir / cdiff))
@@ -1006,7 +1120,7 @@ class CVDUpdate:
                 self.logger.debug(f"An exception occured: {exc}")
                 self.logger.error(f"Failed to delete {cdiff} from databse directory!")
 
-        self.config['dbs'].pop(db)
+        self.state['dbs'].pop(db)
 
         self.logger.info(f"Removed {db} from DB list.")
 
